﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Noise;
using Verse.Sound;

namespace SimpleLootBox
{
    public class CompLootBox : CompUseEffect
    {
        public CompProperties_LootBox Props => (CompProperties_LootBox)props;

        public override void DoEffect(Pawn usedBy)
        {
            base.DoEffect(usedBy);
            LootBoxOpeningWindow window = new LootBoxOpeningWindow(this);
            Find.WindowStack.Add(window);
        }

        public bool Spawn(LootBoxSpinner.SpinItem reward)
        {
            if (reward.count <= 0) return false;

            Map map = parent.Map;
            IntVec3 position = parent.Position;

            if (reward.thingDef != null)
            {
                for (int i = 0; i < reward.count; i++)
                {
                    Thing thing;

                    if (reward.thingDef.MadeFromStuff && reward.stuff != null)
                    {
                        thing = ThingMaker.MakeThing(reward.thingDef, reward.stuff);
                    }
                    else
                    {
                        thing = ThingMaker.MakeThing(reward.thingDef);
                    }

                    if (thing.TryGetComp<CompQuality>() != null)
                    {
                        thing.TryGetComp<CompQuality>().SetQuality(reward.quality, ArtGenerationContext.Outsider);
                    }

                    if (thing.def.Minifiable)
                    {
                        Thing minifiedThing = MinifyUtility.MakeMinified(thing);
                        GenPlace.TryPlaceThing(minifiedThing, parent.Position, parent.Map, ThingPlaceMode.Near);
                    }
                    else
                    {
                        GenPlace.TryPlaceThing(thing, parent.Position, parent.Map, ThingPlaceMode.Near);
                    }

                    if (reward.effecterDef != null)
                    {
                        Effecter effecter = reward.effecterDef.Spawn();
                        effecter.Trigger(new TargetInfo(position, map), new TargetInfo(position, map));
                        effecter.Cleanup();
                    }
                }
            }
            else if (reward.pawnKindDef != null)
            {
                for (int i = 0; i < reward.count; i++)
                {
                    PawnGenerationRequest request = new PawnGenerationRequest(
                        kind: reward.pawnKindDef,
                        context: PawnGenerationContext.NonPlayer,
                        canGeneratePawnRelations: false,
                        colonistRelationChanceFactor: 0f,
                        forceGenerateNewPawn: true
                    );
                    Pawn pawn = PawnGenerator.GeneratePawn(request);
                    //Some pawns like mechaoid have no faction, so no null check
                    pawn.SetFaction(Faction.OfPlayer);
                    if (pawn.ideo != null)
                    {
                        pawn.ideo.SetIdeo(Faction.OfPlayer.ideos.PrimaryIdeo);
                    }
                    GenSpawn.Spawn(pawn, position, map, WipeMode.Vanish);

                    if (reward.effecterDef != null)
                    {
                        Effecter effecter = reward.effecterDef.Spawn();
                        effecter.Trigger(new TargetInfo(position, map), new TargetInfo(position, map));
                        effecter.Cleanup();
                    }
                }
            }
            return true;
        }

        public void DeleteBox(int count)
        {
            if (parent.stackCount > count)
            {
                parent.stackCount -= count;
            }
            else if (parent.stackCount == count)
            {
                parent.Destroy(DestroyMode.Vanish);
            }
        }
    }
}
